/*****************************************************************************************************************************
--
--    File Name    : ddr_write_contrl_ch2.v 

--    Description  : This module implements DDR write controller for write channels 1 and 2
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module ddr_write_contrl #(parameter g_AXI_AWIDTH = 32,
                          parameter g_AXI_BUFF_AWIDTH = 12)
                        (input wire clock,
                         input wire nreset,
                         input wire wr_req_i,
                         output reg wr_ack_o,
                         output reg wr_done_o,
                         input wire[(g_AXI_AWIDTH - 1) : 0] ddr_write_start_address_i,
                         input wire[(g_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_write_i,
                         output wire[4 : 0] axi_burst_len,
                         input wire axi_tr_done,
                         output reg[(g_AXI_AWIDTH - 1) : 0] axi_wr_ddr_addr,
                         output reg start_axi_tr,
                         output wire last_burst_o,
                         output reg start_pack,
                         output reg wr_burst_split);
  reg[(g_AXI_AWIDTH - 1) : 0] II0;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] lI0;
  reg[4 : 0] Il0;
  reg[4 : 0] ll0;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] O10;
  reg[(g_AXI_BUFF_AWIDTH + 3) - 1 : 0] il0;
  reg last_transact;
  reg l10;
  reg O00;
  reg I10;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] o00;
  reg[(g_AXI_BUFF_AWIDTH - 1) : 0] i00;
  reg o10;
  reg[(g_AXI_AWIDTH - 1) : 0] i10;
  reg Oo0;
  parameter[31 : 0] oo0 = 32'd128;
  parameter[4 : 0] io0 = 5'd16;
  parameter[4 : 0] Oi0 = 5'd8;
  parameter[11 : 0] Ii0 = 12'hfff;
  parameter[3 : 0] OO0I = 4'd0,
                   oi0 = 4'd1,
                   ii0 = 4'd2,
                   OO1 = 4'd3,
                   IO1 = 4'd4,
                   lO1 = 4'd5,
                   oO1 = 4'd6,
                   iO1 = 4'd7,
                   OI1 = 4'd8,
                   II1 = 4'd9,
                   lI1 = 4'd10,
                   iI1 = 4'd11,
                   Ol1 = 4'd12,
                   Il1 = 4'd13,
                   O01 = 4'd14,
                   I01 = 4'd15;
  reg[3 : 0] IO0I;
  assign axi_burst_len = Il0;
  assign last_burst_o = last_transact;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          IO0I <= OO0I;
        end
      else
        begin
          case (IO0I)
            OO0I:
              begin
                if (wr_req_i)
                  IO0I <= oi0;
                else
                  IO0I <= OO0I;
              end
            oi0:
              begin
                IO0I <= ii0;
              end
            ii0:
              begin
                IO0I <= OO1;
              end
            OO1:
              begin
                IO0I <= IO1;
              end
            IO1:
              begin
                IO0I <= lO1;
              end
            lO1:
              begin
                IO0I <= oO1;
              end
            oO1:
              begin
                if (l10)
                  begin
                    if (I10)
                      begin
                        if (O00)
                          IO0I <= Il1;
                        else
                          IO0I <= iI1;
                      end
                    else
                      IO0I <= II1;
                  end
                else
                  begin
                    IO0I <= iO1;
                  end
              end
            iO1:
              begin
                IO0I <= OI1;
              end
            OI1:
              begin
                if (I10)
                  begin
                    if (O00)
                      begin
                        IO0I <= I01;
                      end
                    else
                      begin
                        IO0I <= iI1;
                      end
                  end
                else
                  IO0I <= II1;
              end
            II1:
              begin
                if (O00)
                  IO0I <= lI1;
                else
                  begin
                    if (axi_tr_done)
                      begin
                        IO0I <= lI1;
                      end
                    else
                      IO0I <= II1;
                  end
              end
            lI1:
              begin
                if (o10)
                  IO0I <= OI1;
                else
                  IO0I <= OO1;
              end
            iI1:
              begin
                if (axi_tr_done)
                  begin
                    if (o10)
                      begin
                        if (Oo0)
                          IO0I <= Il1;
                        else
                          IO0I <= Ol1;
                      end
                    else
                      begin
                        IO0I <= Il1;
                      end
                  end
                else
                  IO0I <= iI1;
              end
            Ol1:
              begin
                IO0I <= OI1;
              end
            I01:
              begin
                IO0I <= Ol1;
              end
            Il1:
              begin
                if (axi_tr_done)
                  IO0I <= O01;
                else
                  IO0I <= Il1;
              end
            O01:
              begin
                IO0I <= OO0I;
              end
            default:
              IO0I <= OO0I;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          axi_wr_ddr_addr <= { g_AXI_AWIDTH { 1'b0 } };
          start_axi_tr <= 1'b0;
          start_pack <= 1'b1;
          lI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          Il0 <= io0;
          II0 <= { g_AXI_AWIDTH { 1'b0 } };
          i10 <= { g_AXI_AWIDTH { 1'b0 } };
          wr_ack_o <= 1'b0;
          wr_done_o <= 1'b0;
          il0 <= { (g_AXI_BUFF_AWIDTH + 3) { 1'b0 } };
          O00 <= 1'b0;
          wr_burst_split <= 1'b0;
          o00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          ll0 <= 5'd0;
          I10 <= 1'b0;
          i00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          l10 <= 1'b0;
          O10 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          o10 <= 1'b0;
          Oo0 <= 1'b0;
          last_transact <= 1'b0;
        end
      else
        begin
          case (IO0I)
            OO0I:
              begin
                axi_wr_ddr_addr <= { g_AXI_AWIDTH { 1'b0 } };
                start_axi_tr <= 1'b0;
                start_pack <= 1'b1;
                lI0 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                Il0 <= io0;
                II0 <= { g_AXI_AWIDTH { 1'b0 } };
                i10 <= { g_AXI_AWIDTH { 1'b0 } };
                wr_ack_o <= 1'b0;
                wr_done_o <= 1'b0;
                O00 <= 1'b1;
                il0 <= { (g_AXI_BUFF_AWIDTH + 3) { 1'b0 } };
                I10 <= 1'b0;
                wr_burst_split <= 1'b0;
                o00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                ll0 <= 5'd0;
                i00 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                l10 <= 1'b0;
                O10 <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                o10 <= 1'b0;
                Oo0 <= 1'b0;
                last_transact <= 1'b0;
              end
            oi0:
              begin
                II0 <= ddr_write_start_address_i;
                il0 <= bytes_to_write_i;
                wr_ack_o <= 1'b1;
                start_pack <= 1'b0;
              end
            ii0:
              begin
                start_pack <= 1'b1;
                wr_ack_o <= 1'b0;
                lI0 <= il0 >> 3;
              end
            OO1:
              begin
                if (lI0 > io0)
                  begin
                    ll0 <= io0;
                    I10 <= 1'b0;
                  end
                else
                  begin
                    ll0 <= lI0[4 : 0];
                    I10 <= 1'b1;
                  end
              end
            IO1:
              begin
                o00 <= Ii0 - II0[11 : 0];
                i00 <= ll0 * Oi0;
              end
            lO1:
              begin
                if (o00 >= (i00 - 1'b1))
                  l10 <= 1'b1;
                else
                  l10 <= 1'b0;
              end
            oO1:
              begin
                if (l10)
                  begin
                    Il0 <= ll0;
                    if (I10 && O00)
                      begin
                        start_axi_tr <= 1'b1;
                        O00 <= 1'b0;
                        axi_wr_ddr_addr <= II0;
                      end
                    else
                      begin
                        start_axi_tr <= start_axi_tr;
                        O00 <= O00;
                        axi_wr_ddr_addr <= axi_wr_ddr_addr;
                      end
                  end
                else
                  begin
                    o00 <= o00 + 12'd1;
                  end
              end
            iO1:
              begin
                i10 <= II0;
                O10 <= o00 >> 3;
              end
            OI1:
              begin
                Il0 <= O10[4 : 0];
                o10 <= 1'b1;
                if (I10 && O00)
                  begin
                    axi_wr_ddr_addr <= i10;
                    start_axi_tr <= 1'b1;
                  end
                else
                  begin
                    axi_wr_ddr_addr <= axi_wr_ddr_addr;
                    start_axi_tr <= 1'b0;
                  end
              end
            II1:
              begin
                if (O00)
                  begin
                    axi_wr_ddr_addr <= II0;
                    start_axi_tr <= 1'b1;
                    O00 <= 1'b0;
                  end
                else
                  begin
                    if (axi_tr_done)
                      begin
                        if (o10)
                          begin
                            axi_wr_ddr_addr <= i10;
                            start_axi_tr <= 1'b1;
                            O00 <= 1'b0;
                            if (Oo0)
                              begin
                                o10 <= 1'b0;
                                wr_burst_split <= 1'b0;
                              end
                            else
                              begin
                                o10 <= o10;
                                wr_burst_split <= 1'b1;
                              end
                          end
                        else
                          begin
                            axi_wr_ddr_addr <= II0;
                            start_axi_tr <= 1'b1;
                            O00 <= 1'b0;
                          end
                      end
                    else
                      begin
                        axi_wr_ddr_addr <= axi_wr_ddr_addr;
                        O00 <= 1'b0;
                        start_axi_tr <= 1'b0;
                      end
                  end
              end
            lI1:
              begin
                start_axi_tr <= 1'b0;
                if (o10)
                  begin
                    i10 <= i10 + o00;
                    O10 <= ll0 - O10;
                    Oo0 <= 1'b1;
                  end
                else
                  begin
                    lI0 <= lI0 - io0;
                    II0 <= II0 + oo0;
                    Oo0 <= 1'b0;
                  end
              end
            iI1:
              begin
                if (axi_tr_done)
                  begin
                    axi_wr_ddr_addr <= II0;
                    start_axi_tr <= 1'b1;
                  end
                else
                  begin
                    axi_wr_ddr_addr <= axi_wr_ddr_addr;
                    start_axi_tr <= 1'b0;
                  end
              end
            Ol1:
              begin
                start_axi_tr <= 1'b0;
                Oo0 <= 1'b1;
                wr_burst_split <= 1'b1;
                II0 <= II0 + o00;
                O10 <= ll0 - O10;
              end
            I01:
              begin
                O00 <= 1'b0;
                start_axi_tr <= 1'b0;
              end
            Il1:
              begin
                start_axi_tr <= 1'b0;
                wr_burst_split <= 1'b0;
                Oo0 <= 1'b0;
                o10 <= 1'b0;
                last_transact <= 1'b1;
              end
            O01:
              begin
                last_transact <= 1'b0;
                wr_done_o <= 1'b1;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
