/*****************************************************************************************************************************
--
--    File Name    : cdcfiforam.v

--    Description  : This module implements CDC FIFO RAM                                  

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1 ns / 1 ps  

module fifocdcram (data, wren, wraddress, rdaddress, wrclock, rdclock, q);

   parameter ADDR_WIDTH  = 4;
   parameter DATA_WIDTH  = 32;
   input[(DATA_WIDTH - 1):0] data;
   input wren;
   input[(ADDR_WIDTH - 1):0] wraddress;
   input[(ADDR_WIDTH - 1):0] rdaddress;
   input wrclock;
   input rdclock;
   output[(DATA_WIDTH - 1):0] q;
   wire[(DATA_WIDTH - 1):0] q;

   reg[DATA_WIDTH - 1:0] ram_block[0:2 ** ADDR_WIDTH - 1]/* synthesis syn_ramstyle = lsram */;
   reg[ADDR_WIDTH - 1:0] rdaddr_r;
   reg[ADDR_WIDTH - 1:0] wraddr_r;
   reg wren_r;
   reg[DATA_WIDTH - 1:0] wrdata_r;

   // Write process
   always @(posedge wrclock)
   begin
      wren_r <= wren ;
      wraddr_r <= wraddress ;
      wrdata_r <= data ;
      if (wren_r == 1'b1)
      begin
         ram_block[wraddr_r] <= wrdata_r ;
      end
   end

   // Read process
   always @(posedge rdclock)
   begin
      rdaddr_r <= rdaddress ;
   end
   assign q = ram_block[rdaddr_r] ;
endmodule
